<?php

/*

    GTPhoto2 - A multi user photo album
    Copyright (C) 2004-2006 Chris Kloosterman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

	# constants
	$text_size = 8;
	$title_size = 12;

	require_once("framework/includes.php");

	$graph = $_SESSION["graph"][$_REQUEST["id"]];

	if (!is_array($graph)) {
		puke("Error loading graph data");
	}

	# determine the overall image size
	$x_size = $graph["metadata"]["xsize"];
	# left side
	$left = 50;
	# right side
	$right = 20;

	$x_size += $left;
	$x_size += $right;
	
	$y_size = $graph["metadata"]["ysize"];
	# top
	$top = 0;
	## space
	$top += 5;
	## title
	$top += $title_size;
	## space
	$top += 8;

	$y_size += $top;

	# bottom
	$bottom = 0;
	## space
	$bottom += 5;
	## legend vs. timing stats
	$legend = 0;
	$timing = 0;
	if ($graph["metadata"]["show_legend"]) {
		$legend .= (count($graph) - 1) * ($text_size + 5);
	}
	if ($graph["metadata"]["show_proctime"]) {
		$timing += $text_size + 2;
	}
	if ($graph["metadata"]["show_gentime"]) {
		$timing += $text_size + 2;
	}

	if ($legend > $timing) {
		$bottom += $legend;
	} else {
		$bottom += $timing;
	}
	## space
	$bottom += 8;

	$y_size += $bottom;

	# make the image we're going to use
	$g = imagecreatetruecolor($x_size, $y_size);

	# set the image parameters
	imageantialias($g, true);

	# set up the colours
	foreach ($graph as $key => $value) {
		if (is_numeric($key)) {
			$colour[$key] = imagecolorallocate($g, $value["colour"]["r"], $value["colour"]["g"], $value["colour"]["b"]);
		}
	}
	$colour["line"] = imagecolorallocate($g, 0, 0, 0);
	$colour["text"] = imagecolorallocate($g, 0, 0, 0);
	$colour["white"] = imagecolorallocate($g, 255, 255, 255);
	$colour["bg"] = imagecolorallocate($g, $graph["metadata"]["bgcolour"]["r"], $graph["metadata"]["bgcolour"]["g"], $graph["metadata"]["bgcolour"]["b"]);

	# draw the basic graph structure
	
	## white background
	imagefill($g, 0, 0, $colour["white"]);
	
	## graph border
	imageline($g, $left, $top, $left, $y_size - $bottom, $colour["line"]);
	imageline($g, $left, $top, $x_size - $right, $top, $colour["line"]);
	imageline($g, $left, $y_size - $bottom, $x_size - $right, $y_size - $bottom, $colour["line"]);
	imageline($g, $x_size - $right, $top, $x_size - $right, $y_size - $bottom, $colour["line"]);
	
	## graph backdrop
	imagefill($g, $left + 1, $top + 1, $colour["bg"]);

	## title
	### determine width of text
	$bbox = imagettfbbox($title_size, 0, GD_FONT, $graph["metadata"]["title"]);
	$text_width = $bbox[2] - $bbox[0];
	### determine the offset from the left side of the bounding box
	$text_offset = ($graph["metadata"]["xsize"] - $text_width) / 2;
	### draw the title
	imagettftext($g, $title_size, 0, $left + $text_offset, 5 + $title_size, $colour["text"], GD_FONT, $graph["metadata"]["title"]);

	# draw the graphs
	foreach ($graph as $key => $value) {
		if (is_numeric($key)) {
			# this is a series to plot
			$x_values = count($value["data"]);
			$x_width = $graph["metadata"]["xsize"] / ($x_values - 1);
			$y_range = $graph["metadata"]["ymax"] - $graph["metadata"]["ymin"];
			# need to start the graph somewhere
			$prev_x = $left;
			$prev_y = ((1 - ($value["data"][0] - $graph["metadata"]["ymin"]) / $y_range) * $graph["metadata"]["ysize"]) + $top;
			# plot each point
			for ($i = 1; $i < $x_values; $i++) {
				$new_x = $left + ($x_width * $i);
				$new_y = ((1 - ($value["data"][$i] - $graph["metadata"]["ymin"]) / $y_range) * $graph["metadata"]["ysize"]) + $top;

				imageline($g, $prev_x, $prev_y, $new_x, $new_y, $colour[$key]);

				$prev_x = $new_x;
				$prev_y = $new_y;
			}
		}
	}
	
	# finish off the structure

	## legend
	if ($graph["metadata"]["show_legend"]) {
	
		# determine the maximum text width
		$max_legend = 0;
		foreach ($graph as $key => $value) {
			if (is_numeric($key)) {
				$bbox = imagettfbbox($text_size, 0, GD_FONT, $value["legend"]);
				$legend_width = $bbox[2] - $bbox[0];
				if ($legend_width > $max_legend) {
					$max_legend = $legend_width;
				}
			}
		}
		
		# print the legends
		$count = 0;
		foreach ($graph as $key => $value) {
			if (is_numeric($key)) {
				imagettftext($g, $text_size, 0, $x_size - $right - $max_legend, $y_size - $bottom + 8 + $count * ($text_size + 5) + $text_size, $colour["text"], GD_FONT, $value["legend"]);
				imageline($g, $x_size - $right - $max_legend - 20, $y_size - $bottom + 8 + $count * ($text_size + 5) + 0.5 * $text_size, $x_size - $right - $max_legend - 5, $y_size - $bottom + 8 + $count * ($text_size + 5) + 0.5 * $text_size, $colour[$key]);
				$count++;
			}
		}
		
	}

	## gridlines and y axis
	$gridline_height = $graph["metadata"]["ysize"] / ($graph["metadata"]["show_gridlines"] + 1);
	$gridline_adder = ($graph["metadata"]["ymax"] - $graph["metadata"]["ymin"]) / ($graph["metadata"]["show_gridlines"] + 1);
	
	# the top axis label
	imagettftext($g, $text_size, 0, 5, $top + ($text_size / 2), $colour["text"], GD_FONT, $graph["metadata"]["ymax"]);
	
	# the gridline labels
	for ($i = 1; $i <= $graph["metadata"]["show_gridlines"]; $i++) {
		if ($graph["metadata"]["show_gridlines"] > 0) {
			imageline($g, $left, $top + $i * $gridline_height, $x_size - $right, $top + $i * $gridline_height, $colour["line"]);
		}
		$y_value = $graph["metadata"]["ymax"] - ($gridline_adder * $i);
		$y_value = round($y_value * 100) / 100;
		imagettftext($g, $text_size, 0, 5, $top + ($i * $gridline_height) + ($text_size / 2), $colour["text"], GD_FONT, $y_value);
	}
	
	# the bottom axis label
	imagettftext($g, $text_size, 0, 5, $top + ($gridline_height * ($graph["metadata"]["show_gridlines"] + 1)) + ($text_size / 2), $colour["text"], GD_FONT, $graph["metadata"]["ymin"]);

	## redo the graph border to fix where it has been
	## slightly overdrawn by the graph lines
	imageline($g, $left, $top, $left, $y_size - $bottom, $colour["line"]);
	imageline($g, $left, $top, $x_size - $right, $top, $colour["line"]);
	imageline($g, $left, $y_size - $bottom, $x_size - $right, $y_size - $bottom, $colour["line"]);
	imageline($g, $x_size - $right, $top, $x_size - $right, $y_size - $bottom, $colour["line"]);

	## generation time, processing time
	if ($graph["metadata"]["show_gentime"]) {
		$time = date("F j, Y H:i T");
		imagettftext($g, $text_size, 0, $left, $y_size - $bottom + 8 + $text_size, $colour["text"], GD_FONT, "Generated $time");
		$offset_add = $text_size + 5;
	}
	
	if ($graph["metadata"]["show_proctime"]) {
		global $PAGE_START_TIME;
		$PAGE_END_TIME = get_microtime();
		$PAGE_DRAW_TIME = $PAGE_END_TIME - $PAGE_START_TIME;
		
		$time = round($PAGE_DRAW_TIME * 1000) / 1000;
		
		imagettftext($g, $text_size, 0, $left, $y_size - $bottom + 8 + $text_size + $offset_add, $colour["text"], GD_FONT, "Processing time: $time seconds");
	}
	
	# output the image
	header("Content-Type: image/png");
	imagepng($g);

	# now get rid of the image we generated
	imagedestroy($g);

?>
